/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtWidgets>

#include "stareditor.h"
#include "starrating.h"

#include <cmath>

StarEditor::StarEditor(const QModelIndex &index, QWidget *parent)
	: QWidget(parent)
{
	starRating = index.data().value<StarRating>();
	_index = index;
	this->setMouseTracking(true);
	this->setFocusPolicy(Qt::ClickFocus);
	this->setFocus();
	parent->installEventFilter(this);
}

void StarEditor::mouseMoveEvent(QMouseEvent *event)
{
	double starWidth = this->rect().height();
	starRating.setStarCount(std::floor(event->x() / starWidth + 0.5));
	update();
}

void StarEditor::mousePressEvent(QMouseEvent *event)
{
	if (rect().contains(event->pos())) {
		StarRating savedRating = _index.data().value<StarRating>();
		// Send signal only if one has really changed the value of the track
		if (starRating.starCount() == savedRating.starCount()) {
			close();
			return;
		} else {
			emit editFinished();
		}
	}
	QWidget::mousePressEvent(event);
}

void StarEditor::paintEvent(QPaintEvent *)
{
	QStylePainter painter(this);
	QStyleOptionViewItem o;
	o.rect = rect();
	o.palette = this->palette();
	starRating.paintStars(&painter, o, StarRating::Editable);
}
